/*
 * This file is part of libportable.
 * Copyright (C) 2008-2009 David Sveningsson <ext@sidvind.com>
 * 
 * libportable is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * libportable is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with libportable.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PORTABLE_DIRECTORY_H
#define __PORTABLE_DIRECTORY_H

#include <vector>

namespace Portable {
	class Filter {
	    public:
	        typedef std::vector<const char*> __vector;
	        typedef __vector::iterator iterator;
	        typedef __vector::const_iterator const_iterator;

	        Filter();
	        Filter(const char* f, ...);
	        ~Filter();

	        bool match(const char* str);

	        iterator begin() { return _filter.begin(); }
	        iterator end() { return _filter.end(); }
	        const_iterator begin() const { return _filter.begin(); }
	        const_iterator end() const { return _filter.end(); }
	    private:
	        __vector _filter;
	};

	class Directory {
	    public:
	        typedef std::vector<char*> __vector;
	        typedef __vector::iterator iterator;
	        typedef __vector::const_iterator const_iterator;

	        Directory(const char* path);
	        Directory(const char* path, Filter filter);
	        ~Directory();

	        size_t size(){ return _content.size(); }

	        iterator begin() { return _content.begin(); }
	        iterator end() { return _content.end(); }
	        const_iterator begin() const { return _content.begin(); }
	        const_iterator end() const { return _content.end(); }

	    private:
	        void fill(const char* path);

	        __vector _content;
	        Filter _filter;
	};

	bool file_readable(const char* fullpath);
	bool file_writable(const char* fullpath);
}

#endif //__PORTABLE_DIRECTORY_H
