/*
 * This file is part of libportable.
 * Copyright (C) 2008-2009 David Sveningsson <ext@sidvind.com>
 * 
 * libportable is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * libportable is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with libportable.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PORTABLE_TIME_H
#define __PORTABLE_TIME_H

#include <portable/Types.h>

/**
 * @brief Initializes the time subsystem
 */
void initTime();

/**
 * @brief Get current timestamp
 * This is the time since the application was started (since initTime was run to be exact).
 *
 * @return A double with the current timestamp. 1.0f == 1 second.
 */
double getTime();
float getTimef();

/**
 * @brief Sleep
 */
void wait(uint32_t ms);

/**
 * @brief Sleep
 */
void wait(double t);

/**
 * Like gmtime but stores the result in a user-supplied struct.
 * Fallback implementation.
 */
extern "C" struct tm* gmtime_r(const time_t* timep, struct tm* result);

#endif //__PORTABLE_TIME_H
