/*
 * This file is part of libportable.
 * Copyright (C) 2008-2009 David Sveningsson <ext@sidvind.com>
 * 
 * libportable is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * libportable is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with libportable.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include <portable/Directory.h>
#include <stdexcept>
#include <cstring>
#include <cstdlib>
#include <stdarg.h>

#ifdef __GNUC__
#	include <dirent.h>
#	include <sys/types.h>
#	include <sys/stat.h>
#	include <unistd.h>
#endif

#include <limits.h>

#ifdef HAVE_FEATURES_H
#	include <features.h>
#endif

#if defined(__GLIBC__) && __GLIBC__ == 2
#	if defined (__GLIBC_MINOR__) && __GLIBC_MINOR__ < 8 
#		define __OLD_GLIBC__
#	endif
#endif

namespace Portable {
	Filter::Filter(){

	}

	Filter::Filter(const char* f, ...){
		_filter.push_back(f);

		va_list va;
		va_start(va, f);

		const char* str = va_arg(va, const char*);
		while ( str ){
			_filter.push_back(str);
			str = va_arg(va, const char*);
		}

		va_end(va);
	}

	Filter::~Filter(){

	}

	bool Filter::match(const char* str){
		for ( iterator it = _filter.begin(); it != _filter.end(); ++it ){
			if ( strcmp(str, *it) == 0 ){
				return true;
			}
		}

		return false;
	}

	Directory::Directory(const char* path){
		fill(path);
	}

	Directory::Directory(const char* path, Filter filter): _filter(filter){
		fill(path);
	}


	Directory::~Directory(){

	}

	static Filter* gs_filter = NULL;

#if !defined(__GLIBC__) || defined(__OLD_GLIBC__)
	static int filter(struct dirent* file){
#else
	static int filter(const struct dirent* file){
#endif
		if ( gs_filter->match(file->d_name) ){
			return 0;
		}
		return 1;
	}

	void Directory::fill(const char* path){
	#ifdef __GNUC__
		struct dirent** list;
		int n;

		gs_filter = &_filter;
		n = scandir(path, &list, filter, alphasort);
		if ( n < 0 ){
			goto error_handler;
		}

		for ( int i = 0; i < n; i++ ){
			_content.push_back( list[i]->d_name);
		}

		free(list);
	#endif

		return;

		error_handler:
		throw std::runtime_error("Failed to open directory");
	}

	bool file_readable(const char* fullpath){
#ifdef __GNUC__
		struct stat st;
		if( stat(fullpath, &st) != 0 ){
			return false;
		}

		if( access(fullpath, R_OK) == 0 ){
			return true;
		}

		return false;
#else
#	error Not implemented
#endif
	}

	bool file_writable(const char* fullpath){
#ifdef __GNUC__
		if( access(fullpath, W_OK) == 0 ){
			return true;
		}
		return false;
#else
#	error Not implemented
#endif
	}
}
