/*
 * This file is part of libportable.
 * Copyright (C) 2008-2009 David Sveningsson <ext@sidvind.com>
 * 
 * libportable is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * libportable is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with libportable.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include <sys/types.h>

#ifdef __GNUC__
#	include <unistd.h>
#endif

#include <cstdio>
#include <cstdlib>
#include <stdexcept>

#include "portable/Directory.h"

namespace Portable {

	void daemonize(const char* name){
#ifdef __GNUC__
		char* filename;
		asprintf(&filename, "%s.pid", name);
#else
#	error Not implemented
#endif

		// Check if a pid-file already exists
		if ( file_readable(filename) ){
			FILE* f = fopen(filename, "r");
			int pid;
			fscanf(f, "%d\n", &pid);
			fclose(f);

#ifdef __GNUC__
			char* error_message;
			asprintf(&error_message, "%s (pid %d) is already running\n", name, pid);
#else
			char error_message[512];
			snprintf(error_message, 512, "%s (pid %d) is already running\n", name, pid);
#endif

			throw std::runtime_error(error_message);
		}

#ifdef __GNUC__
		int pid = fork();
		if ( pid != 0){

			FILE* f = fopen(filename, "w");
			fprintf(f, "%d\n", pid);
			fclose(f);

			free(filename);

			exit(0);
		}
#else
#	error Not implemented
#endif

	}

	void daemon_stop(const char* name){
#ifdef __GNUC__
		char* filename;
		asprintf(&filename, "%s.pid", name);
#else
#	error Not implemented
#endif

		// Check if a pid-file exists
		if ( !file_readable(filename) ){
			throw std::runtime_error("Daemon not running (no pid file)");
		}

#ifdef __GNUC__
		unlink(filename);
#else
#	error Not implemented
#endif
	}

}
