/*
 * This file is part of libportable.
 * Copyright (C) 2008-2009 David Sveningsson <ext@sidvind.com>
 * 
 * libportable is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * libportable is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with libportable.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include <portable/asprintf.h>

#ifndef HAVE_ASPRINTF
#include <stdio.h>
#include <stdlib.h>

/* On WinCE vsnprintf has been named _vsnprintf and if using MSVC2005 or later
 * the security enhanched version _vsnprintf_s is used. */
#if defined(WINCE)
#	define vsnprintf _vsnprintf
#elif _MSC_VER >= 1400
#	define vsnprintf(dst, size, fmt, ap) _vsnprintf_s(dst, size, _TRUNCATE, fmt, ap)
#endif

static char* alloc(char* buf, size_t size);

int asprintf(char **strp, const char *fmt, ...){
	va_list ap;
	int ret;
	
	va_start(ap, fmt);
	ret = vasprintf(strp, fmt, ap);
	va_end(ap);
	return ret;
}

int vasprintf(char **strp, const char *fmt, va_list ap){
	size_t size = 32;
	char* buf = 0;
	int n = 0;
	
	for (;;) {
		size *= 2;
		buf = alloc(buf, size);
		if ( !buf ){
			return -1;
		}

		n = vsnprintf(buf, size, fmt, ap);

		// @todo @bug the return value is different for msvc compilers and real
		//            compilers so this conditional must be adjusted.
		if ( n < 0 ){
			continue;
		}

		break;
	}
	
	*strp = buf;
	return n;
}

static char* alloc(char* buf, size_t size){
	return (char*)realloc(buf, size);
}

#endif /* HAVE_ASPRINTF */


char* asprintf2(const char* fmt, ...){
	va_list ap;
	char* ret;

	va_start(ap, fmt);
	ret = vasprintf2(fmt, ap);
	va_end(ap);

	return ret;
}

char* vasprintf2(const char* fmt, va_list ap){
	char* tmp = NULL;
	if ( vasprintf(&tmp, fmt, ap) < 0 ){
		return NULL;
	}
	return tmp;
}

#ifdef WIN32

static wchar_t* allocw(wchar_t* buf, size_t size){
	return (wchar_t*)realloc(buf, size * sizeof(wchar_t));
}

int  aswprintf(wchar_t** strp, const wchar_t* fmt, ...){
	va_list ap;
	int ret;
	
	va_start(ap, fmt);
	ret = vaswprintf(strp, fmt, ap);
	va_end(ap);
	return ret;
}

int vaswprintf(wchar_t** strp, const wchar_t* fmt, va_list ap){
	size_t size = 32;
	wchar_t* buf = 0;
	int n = 0;
	
	for (;;) {
		size *= 2;
		buf = allocw(buf, size);
		if ( !buf ){
			return -1;
		}

		n = _vsnwprintf_s(buf, size, _TRUNCATE, fmt, ap);

		// @todo @bug the return value is different for msvc compilers and real
		//            compilers so this conditional must be adjusted.
		if ( n < 0 ){
			continue;
		}

		break;
	}
	
	*strp = buf;
	return n;
}

wchar_t*  aswprintf2(wchar_t* fmt, ...){
	va_list ap;
	wchar_t* ret;

	va_start(ap, fmt);
	ret = vaswprintf2(fmt, ap);
	va_end(ap);

	return ret;
}

wchar_t* vaswprintf2(wchar_t* fmt, va_list ap){
	wchar_t* tmp = NULL;
	if ( vaswprintf(&tmp, fmt, ap) < 0 ){
		return NULL;
	}
	return tmp;
}

#endif /* WIN32 */
