/*
 * This file is part of libportable.
 * Copyright (C) 2008-2009 David Sveningsson <ext@sidvind.com>
 * 
 * libportable is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * libportable is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with libportable.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include <portable/Time.h>

#if !(defined(HAVE_GETTIMEOFDAY) || defined(HAVE_QUERYPERFORMANCECOUNTER) || defined(HAVE_GETTICKCOUNT))
#	error No function to get time
#endif

#ifdef __GNUC__
#	include <sys/time.h>
#	include <unistd.h>
#endif

#if defined(WIN32) || defined(WINCE)
#	include "_windows.h"
#endif

#ifndef NULL
#   define NULL 0
#endif

#if defined(HAVE_GETTIMEOFDAY)
	static double s_InitTime;
#elif defined(HAVE_QUERYPERFORMANCECOUNTER)
	static LARGE_INTEGER s_InitTicks;
	static double s_invFrequency;
#elif defined(HAVE_GETTICKCOUNT)

#endif

#ifdef HAVE_GETTIMEOFDAY
static inline double timeval_to_double( const struct timeval& tval ){
	return static_cast<double>(tval.tv_sec) + static_cast<double>(tval.tv_usec) * 1e-6;
}
#endif // __GNUC__

void initTime(){
#if defined(HAVE_GETTIMEOFDAY)
	timeval tval;
	gettimeofday(&tval, NULL);
	s_InitTime=timeval_to_double(tval);
#elif defined(HAVE_QUERYPERFORMANCECOUNTER)
	/* @todo Should use SetThreadAffinityMask to prevent issues with multicore
	 *       and multiprocessors. */
	QueryPerformanceCounter(&s_InitTicks);
	LARGE_INTEGER tmp;
	QueryPerformanceFrequency(&tmp);
	s_invFrequency = 1.0 / tmp.QuadPart;
#elif defined(HAVE_GETTICKCOUNT)
#	error Not implemented
#endif
}

double getTime(){
#if (HAVE_GETTIMEOFDAY)
	timeval tval;
	gettimeofday(&tval, NULL);

	double time = timeval_to_double(tval);

	return time - s_InitTime;
#elif defined(HAVE_QUERYPERFORMANCECOUNTER)
	/* @todo Should use SetThreadAffinityMask to prevent issues with multicore
	 *       and multiprocessors. */
	LARGE_INTEGER tmp;
	QueryPerformanceCounter(&tmp);
	return (tmp.QuadPart - s_InitTicks.QuadPart) * s_invFrequency;
#elif defined(HAVE_GETTICKCOUNT)
#	error Not implemented
#endif
}

float getTimef(){
	return static_cast<float>(getTime());
}

void wait(uint32_t ms){
#ifdef __GNUC__
	usleep(ms*1000);
#else
	Sleep(ms);
#endif
}

void wait(double t){
#ifdef __GNUC__
	usleep( (int)(t * 1000 * 1000) );
#else
	Sleep( (int)(t * 1000) );
#endif
}
